# **************************************************************************
# *
# * Authors:     J.M. De la Rosa Trevin (jmdelarosa@cnb.csic.es)
# *              Roberto Marabini (roberto@cnb.csic.es)
# *
# * Unidad de  Bioinformatica of Centro Nacional de Biotecnologia , CSIC
# *
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation; either version 3 of the License, or
# * (at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program; if not, write to the Free Software
# * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
# * 02111-1307  USA
# *
# *  All comments concerning this program package may be sent to the
# *  e-mail address 'scipion@cnb.csic.es'
# *
# **************************************************************************

from matplotlib.ticker import FuncFormatter
from matplotlib.widgets import Button

from pyworkflow import Config
import pyworkflow.viewer as pwviewer

import pwem.objects as emobj
import pwem.protocols as emprot

from .plotter import EmPlotter, plt


class FscViewer(pwviewer.Viewer):
    """ Viewer for plot a FSC object. """
    _targets = [emobj.FSC, emobj.SetOfFSCs]
    _environments = [pwviewer.DESKTOP_TKINTER, pwviewer.WEB_DJANGO]

    def __init__(self, **kwargs):
        pwviewer.Viewer.__init__(self, **kwargs)
        self.threshold = kwargs.get('threshold', 0.143)
        self.legend = ""
        self.plotter = EmPlotter(x=1, y=1, windowTitle='FSC',
                                 figure=kwargs.get('figure', 'active'))
        self._lastSubPlot = None
        self._addButton = kwargs.get('addButton', False)

    def _formatFreq(self, value, pos):
        """ Format function for Matplotlib formatter. """
        inv = 999.
        if value:
            inv = 1 / value
        return "1/%0.2f" % inv

    def show(self, block=True):
        self.plotter.show(block=block)

    def _plotButton(self):
        if self._addButton:
            axcreateFSC = plt.axes([0.75, 0.02, 0.2, 0.050])
            # Button does not allow to define text color so
            # I write it directly
            axcreateFSC.text(0.5, 0.5, 'create FSC',
                             verticalalignment='center',
                             horizontalalignment='center',
                             transform=axcreateFSC.transAxes, color='white')
            bcreateFSC = Button(axcreateFSC, '',  # leave label empty
                                color=Config.SCIPION_MAIN_COLOR,
                                hovercolor='maroon')
            bcreateFSC.on_clicked(self.createFSCObject)
            self._addButton = False
            return bcreateFSC

    def visualize(self, obj, **kwargs):
        # Keep input object in case we need to launch
        # a new protocol and set dependencies
        self.fscList = []
        if isinstance(obj, emobj.SetOfFSCs):
            for i, fsc in enumerate(obj):
                self.fscList.append(fsc.clone())
                fscLabel = fsc.getObjLabel() or 'FSC %d' % (i + 1)
                self.plotFsc(fsc, label=fscLabel, **kwargs)
        else:  # single FSC
            fscLabel = obj.getObjLabel() or 'FSC 1'
            self.plotFsc(obj, label=fscLabel, **kwargs)
            self.fscList.append(obj.clone())

        bcreateFSC = self._plotButton()  # if you do not assign the result
        # to something it will be
        # delete
        self.show()
        # return [self.plotter]

    def createFSCObject(self, event):
        self.project = self.getProject()
        prot = self.getProject().newProtocol(emprot.ProtCreateFSC)
        prot.setObjLabel('FSC-%s' % self.label)  # label)
        prot.setInputObj(self.protocol)
        prot.setInputFscList(self.fscList)
        self.project.launchProtocol(prot)

    def plotFsc(self, obj, **kwargs):

        x, y = obj.getData()

        if self._lastSubPlot is None:
            a = self.plotter.createSubPlot("FSC", "frequency (1/A)", "fsc")
            a.set_ylim([-0.1, 1.1])
            a.plot([0, x[-1]],
                   [self.threshold, self.threshold],
                   'k--')
            a.grid(True)
            a.xaxis.set_major_formatter(FuncFormatter(self._formatFreq))
            self._lastSubPlot = a
        resolution = obj.calculateResolution(self.threshold)
        self.label = kwargs.get('label', self.protocol.getRunName()) + " (" + str(resolution) + "Å)"
        self.plotter.plotData(x, y, '-', label=self.label)
        self.plotter.legend()
        # if I run this



# ***************************************************************************
# * Authors:     Laura del Cano (ldelcano@cnb.csic.es)
# *
# *
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation; either version 3 of the License, or
# * (at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program; if not, write to the Free Software
# * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
# * 02111-1307  USA
# *
# *  All comments concerning this program package may be sent to the
# *  e-mail address 'scipion@cnb.csic.es'
# ***************************************************************************/

import os

import pyworkflow.utils as pwutils
import pyworkflow.tests as pwtests

import pwem.protocols as emprot


class TestImportCTFs(pwtests.BaseTest):
    @classmethod
    def setUpClass(cls):
        pwtests.setupTestProject(cls)
        cls.dsXmipp = pwtests.DataSet.getDataSet('xmipp_tutorial')
        cls.dsGrigorieff = pwtests.DataSet.getDataSet('grigorieff')
        cls.dsEman2 = pwtests.DataSet.getDataSet('eman')

        # First, import a set of micrographs that will be used
        # from all ctf test cases
        cls.protImport = cls.newProtocol(emprot.ProtImportMicrographs,
                                         filesPath=cls.dsXmipp.getFile('allMics'),
                                         samplingRate=1.237, voltage=300)
        cls.launchProtocol(cls.protImport)

    def testImportCTFFromXmipp(self):
        protCTF = self.newProtocol(emprot.ProtImportCTF,
                                   importFrom=emprot.ProtImportCTF.IMPORT_FROM_XMIPP3,
                                   filesPath=self.dsXmipp.getFile('ctfsDir'),
                                   filesPattern='*.ctfparam')
        protCTF.inputMicrographs.set(self.protImport.outputMicrographs)
        protCTF.setObjLabel('import ctfs from xmipp ')
        self.launchProtocol(protCTF)

        self.assertIsNotNone(protCTF.outputCTF,
                             "There was a problem when importing ctfs.")
        
    def testImportCtffind4(self):
        protCTF = self.newProtocol(emprot.ProtImportCTF,
                                   importFrom=emprot.ProtImportCTF.IMPORT_FROM_GRIGORIEFF,
                                   filesPath=self.dsGrigorieff.getFile('ctffind4'),
                                   filesPattern='BPV*/*txt')
        protCTF.inputMicrographs.set(self.protImport.outputMicrographs)
        protCTF.setObjLabel('import from ctffind4')
        self.launchProtocol(protCTF)

        self.assertIsNotNone(protCTF.outputCTF,
                             "There was a problem when importing ctfs.")

    def testImportFromScipion(self):
        ctfSqlite = self.dsGrigorieff.getFile('ctffind3/ctfs.sqlite')

        protCTF = self.newProtocol(emprot.ProtImportCTF,
                                   objLabel='import from scipion',
                                   importFrom=emprot.ProtImportCTF.IMPORT_FROM_SCIPION,
                                   filesPath=ctfSqlite)

        protCTF.inputMicrographs.set(self.protImport.outputMicrographs)
        self.launchProtocol(protCTF)

        self.assertIsNotNone(protCTF.outputCTF,
                             "There was a problem when importing ctfs.")

    def testImportFromCTffind4Conflict(self):
        micsPath = os.path.abspath(self.proj.getTmpPath('micrographs'))
        ctfsPath = os.path.abspath(self.proj.getTmpPath('ctfs'))
        pwutils.makePath(micsPath)
        pwutils.makePath(ctfsPath)

        micDict = {'BPV_1386': 'mic1',
                   'BPV_1387': 'mic10',
                   'BPV_1388': 'mic2'}

        pattern = 'micrographs/%s.mrc'

        for k, v in micDict.items():
            # Create a micrograph link with a given naming convention
            pwutils.createAbsLink(self.dsXmipp.getFile(pattern % k),
                                  self.proj.getTmpPath(pattern % v))

            pwutils.createAbsLink(self.dsGrigorieff.getFile('ctffind4/%s' % k),
                                  self.proj.getTmpPath('ctfs/%s' % v))

        protCTF = self.newProtocol(emprot.ProtImportCTF,
                                   importFrom=emprot.ProtImportCTF.IMPORT_FROM_GRIGORIEFF,
                                   filesPath=ctfsPath,
                                   filesPattern='mic*/*txt')
        protCTF.inputMicrographs.set(self.protImport.outputMicrographs)
        protCTF.setObjLabel('import from ctffind3 - empty')
        self.launchProtocol(protCTF)

        # FIXME: After the execution of the above case, some empty sets
        # are created, if there is no matching, there should not be any output

        # Let's import now the correct names, but with the problematic matching
        protImport2 = self.newProtocol(emprot.ProtImportMicrographs,
                                       filesPath=micsPath,
                                       filesPattern='mic*mrc',
                                       samplingRate=1.237, voltage=300)
        self.launchProtocol(protImport2)

        protCTF2 = self.newProtocol(emprot.ProtImportCTF,
                                    importFrom=emprot.ProtImportCTF.IMPORT_FROM_GRIGORIEFF,
                                    filesPath=ctfsPath,
                                    filesPattern='mic*/*txt')
        protCTF2.inputMicrographs.set(protImport2.outputMicrographs)
        protCTF2.setObjLabel('import from ctffind3 - match')
        self.launchProtocol(protCTF2)

        # The whole set (3 items) should find its corresponding CTF
        self.assertEqual(protCTF2.outputCTF.getSize(), 3)

    def testImportEman2(self):
        protCTF = self.newProtocol(emprot.ProtImportCTF,
                                   importFrom=emprot.ProtImportCTF.IMPORT_FROM_EMAN2,
                                   filesPath=self.dsEman2.getFile('ctfs'),
                                   filesPattern='BPV*json')
        protCTF.inputMicrographs.set(self.protImport.outputMicrographs)
        protCTF.setObjLabel('import from eman2')
        self.launchProtocol(protCTF)

        self.assertIsNotNone(protCTF.outputCTF,
                             "There was a problem when importing ctfs.")



# **************************************************************************
# *
# * Authors:     Roberto Marabini (roberto@cnb.csic.es)
# *              J.M. De la Rosa Trevin (jmdelarosa@cnb.csic.es)
# *
# * Unidad de  Bioinformatica of Centro Nacional de Biotecnologia , CSIC
# *
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation; either version 3 of the License, or
# * (at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program; if not, write to the Free Software
# * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
# * 02111-1307  USA
# *
# *  All comments concerning this program package may be sent to the
# *  e-mail address 'scipion@cnb.csic.es'
# *
# **************************************************************************
from datetime import datetime

import emtable

import pyworkflow.tests as pwtests

import pwem.emlib.metadata as md


class TestMetaData(pwtests.unittest.TestCase):
    
    _labels = [pwtests.WEEKLY]

    def _newMd(self, n=5):
        md0 = md.MetaData()
        xcoor = range(n)
        ycoor = [x*x for x in xcoor]
        for i in range(n):
            self._addRow(md0, '%02d@proj.stk' % i, xcoor[i], ycoor[i])
        return md0

    def _addRow(self, md0, imageFn, xcoor, ycoor):
        objId = md0.addObject()
        md0.setValue(md.MDL_IMAGE, imageFn, objId)
        md0.setValue(md.MDL_XCOOR, xcoor, objId)
        md0.setValue(md.MDL_YCOOR, ycoor, objId)

    def test_removeDuplicates(self):
        md0 = self._newMd()
        md1 = self._newMd()

        # If removing without labels, this metadata should remain the same
        md1.removeDuplicates()
        self.assertEqual(md0, md1)

        # We can use labels for removeDuplicates
        self._addRow(md1, '00@proj.stk', 0, 0)
        md1.removeDuplicates()
        self.assertEqual(md0, md1)

        self._addRow(md1, '06@proj.stk', 0, 0)
        self.assertNotEqual(md0, md1)
        md1.removeDuplicates()
        self.assertNotEqual(md0, md1)
        md1.removeDuplicates(md.MDL_XCOOR)
        self.assertEqual(md0, md1)

        md1.clear()
        self._addRow(md1, '00@proj.stk', 0, 1)
        self._addRow(md1, '00@proj.stk', 0, 2)
        self._addRow(md1, '00@proj.stk', 0, 3)
        md1.removeDuplicates(md.MDL_IMAGE)
        self.assertEqual(md1.size(), 1)
        objId = md1.firstObject()
        self.assertEqual(md1.getValue(md.MDL_YCOOR, objId), 1)

    def test_dropKeepColumns(self):
        md0 = self._newMd()
        md1 = self._newMd()

        self.assertEqual(md0, md1)

        md.dropColumns(md0, md.MDL_XCOOR, md.MDL_YCOOR)
        self.assertEqual(md0.getActiveLabels(), [md.MDL_IMAGE])

        md.keepColumns(md1, "image")
        self.assertEqual(md1.getActiveLabels(), [md.MDL_IMAGE])

        self.assertEqual(md0, md1)

    def iterRowBinding(self, n):
        md0 = self._newMd(n)
        count = 0
        _dt = datetime.now()
        for row in enumerate(md.iterRows(md0)):
            self.assertIsNotNone(row)
            rowValues = row[1]
            image = rowValues.getValue(label='image')
            xcorr = rowValues.getValue(label='xcoor')
            ycorr = rowValues.getValue(label='ycoor')
            self.assertEqual(xcorr*xcorr, ycorr)
            count += 1
        elapsedTime = datetime.now() - _dt
        self.assertTrue(elapsedTime.seconds < 8,
                        msg="Metadata iteration is too slow")
        self.assertEqual(count, n)
        return elapsedTime

    def iterRowsEMTable(self, n):
        cols = ['image', 'xcoor', 'ycoor']
        table = emtable.Table(columns=cols)
        xcoor = range(n)
        ycoor = [x * x for x in xcoor]
        _dict = dict
        for i in range(n):
            _dict = {'image': '%02d@proj.stk' % i,
                     'xcoor': xcoor[i],
                     'ycoor': ycoor[i]}
            table.addRow(*_dict.values())

        starFile = '/tmp/test.star'
        with open(starFile, 'w') as f:
            f.write("# Star file generated with Scipion\n")
            f.write("# version 30001\n")
            table.writeStar(f, tableName='test')

        count = 0
        _dt = datetime.now()

        for row in enumerate(table.iterRows(fileName=starFile, tableName='test')):
            self.assertIsNotNone(row)
            rowValues = row[1]
            image = rowValues.get('image')
            xcorr = rowValues.get('xcoor')
            ycorr = rowValues.get('ycoor')
            self.assertEqual(xcorr * xcorr, ycorr)
            count += 1

        elapsedTime = datetime.now() - _dt
        self.assertTrue(elapsedTime.seconds < 1,
                        msg="Metadata iteration is too slow")
        self.assertEqual(count, n)
        return elapsedTime

    def test_iterRowMetadata(self):
        """
        Check if iterating over N rows using the binding is faster than emtable
        """
        n = 10000  # number of .star file rows
        iterRowBindingElapsedTime = self.iterRowBinding(n)
        iterRowsEMTableElapsedTime = self.iterRowsEMTable(n)

        print("Binding iteration : %s" % iterRowBindingElapsedTime)
        print("Emtable iteration : %s" % iterRowsEMTableElapsedTime)

        self.assertTrue(iterRowsEMTableElapsedTime < iterRowBindingElapsedTime,
                        msg="Iterating over %d rows using the binding is "
                            "faster than using emtable" % n)




# **************************************************************************
# *
# * Authors:     J.M. De la Rosa Trevin (delarosatrevin@scilifelab.se) [1]
# * Authors:     Daniel Del Hoyo (ddelhoyo@cnb.csic.es)
# *
# * [1] SciLifeLab, Stockholm University
# *
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation; either version 3 of the License, or
# * (at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program; if not, write to the Free Software
# * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
# * 02111-1307  USA
# *
# *  All comments concerning this program package may be sent to the
# *  e-mail address 'scipion@cnb.csic.es'
# *
# **************************************************************************

import os
import numpy as np

import pyworkflow.viewer as pwviewer
import pyworkflow.protocol.params as params
import pyworkflow.gui.plotter as plotter
import pyworkflow.gui.dialog as dialog

import pwem.constants as emcts
import pwem.protocols as emprot
from pwem import emlib, splitRange
from pwem.objects import AtomStruct, SetOfAtomStructs
from pwem.viewers.viewer_chimera import mapVolsWithColorkey, Chimera, ChimeraView, generateColorLegend
from pwem.convert.atom_struct import *

from .plotter import EmPlotter, plt

CHIMERA_ERROR = 'Chimera program is not found were it was expected: \n\n{}\n\n' \
                'Either install ChimeraX in this path or install our ' \
                'scipion-em-chimera plugin'.format(Chimera.getProgram())


class LocalResolutionViewer(pwviewer.ProtocolViewer):
    """
    Visualization tools for local resolution results.

    """
    binaryCondition = ('(colorMap == %d) ' % emcts.COLOR_OTHER)

    def __init__(self, *args, **kwargs):
        pwviewer.ProtocolViewer.__init__(self, **kwargs)

    def getImgData(self, imgFile, minMaskValue=0.1, maxMaskValue=None):
        import numpy as np
        # if image ends in .mrc or .map :mrc
        if imgFile[-4:] == ".mrc" or imgFile[-4:] == ".map":
            imgFile += ":mrc"
        img = emlib.image.ImageHandler().read(imgFile)
        imgData = img.getData()
        voldim = (img.getDimensions())[:-1]

        if maxMaskValue:
            imgData = np.ma.masked_where(imgData > maxMaskValue, imgData, copy=False)
        maxRes = np.amax(imgData)

        if minMaskValue:
            imgData = np.ma.masked_where(imgData < minMaskValue, imgData, copy=False)
        minRes = np.amin(imgData)

        return imgData, minRes, maxRes, voldim

    def getSlice(self, index, volumeData):
        return int(index * volumeData.shape[0] / 9)

    def getSliceImage(self, volumeData, sliceNumber, dataAxis):
        if dataAxis == 'y':
            imgSlice = volumeData[:, sliceNumber, :]
        elif dataAxis == 'x':
            imgSlice = volumeData[:, :, sliceNumber]
        else:
            imgSlice = volumeData[sliceNumber, :, :]

        return imgSlice

    def createChimeraScript(self, scriptFile, fnResVol,
                            fnOrigMap, sampRate, numColors=13, lowResLimit=None, highResLimit=None, showAxis=True):

        imageFile = os.path.abspath(fnResVol)

        _, minRes, maxRes, voldim = self.getImgData(imageFile)

        # Narrow the color range to the highest resolution range
        if lowResLimit is None:
            lowResLimit = min(maxRes, minRes + 5)
        if highResLimit is None:
            highResLimit = minRes

        stepColors = splitRange(highResLimit, lowResLimit,
                                splitNum=numColors)
        colorList = plotter.getHexColorList(len(stepColors), self._getColorName())

        fnVol = os.path.abspath(fnOrigMap)

        mapVolsWithColorkey(fnVol,
                            imageFile,
                            stepColors,
                            colorList,
                            voldim,
                            volOrigin=None,
                            step=-1,
                            sampling=sampRate,
                            scriptFileName=scriptFile,
                            bgColorImage='white',
                            showAxis=showAxis)

    def _getColorName(self):
        return self.colorMap.get()


class ChimeraAttributeViewer(pwviewer.ProtocolViewer):
    """ Viewer for attributes of a SetOfAtomStruct or AtomStruct.
    Includes visualization in chimera and in histograms"""
    _label = 'Atomic structure attributes viewer'
    _targets = []
    _environments = [pwviewer.DESKTOP_TKINTER, pwviewer.WEB_DJANGO]

    def __init__(self, **kwargs):
        pwviewer.ProtocolViewer.__init__(self, **kwargs)
        if not chimeraInstalled():
            print(CHIMERA_ERROR)

    def _defineParams(self, form):
      form.addSection(label='Visualization of attributes')
      form.addParam('attrName', params.EnumParam,
                    choices=self._getStructureAttributes(), default=self._getAttributeIndex(self.protocol._ATTRNAME),
                    label='Attribute to display: ',
                    help='Display this attribute of the structure'
                    )
      group = form.addGroup('Attribute histogram')
      group.addParam('displayHistogram', params.LabelParam,
                     label='Display histogram of attribute: ',
                     help='Display a histogram with the count of the attribute.\n'
                          'The color palette, intervals, lowest and highest values can be chosen in the '
                          'parameters below. If the highest and lowest values you input are the same, '
                          'the lowest and higher values in the date are used.'
                     )

      group = form.addGroup('Visualization over sequence')
      group.addParam('chain_name', params.StringParam, default='A',
                     allowsNull=True, label='Chain of interest:',
                     help='Specify the chain of interest (e.g: A)')
      group.addParam('displaySequence', params.LabelParam,
                     label='Display attribute over sequence: ',
                     help='Display a graph witht the values of the selected attribute over the sequence.'
                     )

      group = form.addGroup('Visualization in Chimera')
      group.addParam('displayStruct', params.LabelParam,
                     label='Display structure and color by attribute in Chimera: ',
                     help='Display the AtomStruct, coloured by the attribute.\n'
                          'The color palette, intervals, lowest and highest values can be chosen in the '
                          'parameters below. If the highest and lowest values you input are the same, '
                          'the lowest and highest values in the date are used.'
                     )
      group.addParam('residuesAverages', params.BooleanParam,
                     default=True, label='Average attribute on residues: ',
                     help='In case the recipients are the atoms, their values are averaged on the residues.\n'
                     )

      from pwem.wizards.wizard import ColorScaleWizardBase
      group = form.addGroup('Color settings')
      ColorScaleWizardBase.defineColorScaleParams(group, defaultLowest=0, defaultHighest=2, defaultIntervals=21,
                                                  defaultColorMap='RdBu')

    def _getOutputObject(self):
        '''Return the output object of the protocol (SetOfAtomStruct or AtomStruct)
        To change in other viewers with different output name'''
        return getattr(self.protocol, self.protocol._OUTNAME)

    def getAtomStructObject(self):
        obj = self._getOutputObject()
        if issubclass(type(obj), SetOfAtomStructs):
          _inputStruct = obj.getRepresentative()
          if _inputStruct is None:
              _inputStruct = obj.getFirstItem()
        elif issubclass(type(obj), AtomStruct):
          _inputStruct = obj
        return _inputStruct

    def _getStructureAttributes(self):
        '''Returns a list with the names of the attributes of the output object'''
        obj = self.getAtomStructObject()
        ASH = AtomicStructHandler()
        cifDic = ASH.readLowLevel(obj.getFileName())
        return list(set(cifDic[NAME]))

    def _getAttributeIndex(self, attrName):
        '''Returns a list with the names of the attributes of the output object'''
        names = self._getStructureAttributes()
        return names.index(attrName)


    def _getVisualizeDict(self):
      return {
        'displayStruct': self._showChimera,
        'displayHistogram': self._showHistogram,
        'displaySequence': self._showSequence,
      }
    
    def _showSequence(self, paramName=None):
        prot = self.protocol
        _inputStruct = self.getAtomStructObject()
        cifFile = _inputStruct.getFileName()
        attrName = self.getEnumText('attrName')
        cifDic = AtomicStructHandler().readLowLevel(cifFile)

        names, values, specs = np.array(cifDic[NAME]), np.array(cifDic[VALUE]), np.array(cifDic[SPEC])
        recipient = getStructureRecipient(cifDic, attrName)
        attrValues, attrSpecs = values[names == attrName], specs[names == attrName]
        if recipient == 'atoms':
            attrValues, attrSpecs = getResidueAverage(attrValues, attrSpecs)

        attrPositions, attrChainValues = getResiduePositions(attrSpecs, attrValues, self.chain_name.get())

        attrValues = list(map(float, attrChainValues))
        xs = np.arange(len(attrValues))

        fig = plt.figure()
        ax = fig.add_subplot(1, 1, 1)
        ax.bar(xs, attrValues)
        yloc = plt.MaxNLocator(10)
        ax.yaxis.set_major_locator(yloc)

        plt.xlabel("Sequence position")
        plt.ylabel("{} value".format(attrName))
        plt.title('{} along sequence'.format(attrName))

        plt.show()
    
    def _showChimera(self, paramName=None):
      if chimeraInstalled():
          prot = self.protocol
          chimScript = prot._getExtraPath('chimera_script.py')
          f = open(chimScript, "w")
          f.write("from chimerax.core.commands import run\n")

          f.write("run(session, 'cd %s')\n" % os.getcwd())
          f.write("run(session, 'cofr 0,0,0')\n")  # set center of coordinates

          _inputStruct = self.getAtomStructObject()
          _inputVol = _inputStruct.getVolume()
          cifFile = _inputStruct.getFileName()
          attrName = self.getEnumText('attrName')
          if _inputVol is not None:
            strId = 3
            dim, sampling = _inputVol.getDim()[0], _inputVol.getSamplingRate()

            f.write("run(session, 'open %s')\n" % _inputVol.getFileName())
            x, y, z = _inputVol.getOrigin(force=True).getShifts()
            f.write("run(session, 'volume #%d style surface voxelSize %f')\n" % (1, sampling))
            f.write("run(session, 'volume #%d origin %0.2f,%0.2f,%0.2f')\n" % (1, x, y, z))
            f.write("run(session, 'hide #!%d models')\n" % 1)

          else:
            dim, sampling = 150., 1.
            strId = 2

          tmpFileName = os.path.abspath(prot._getExtraPath("axis_input.bild"))
          Chimera.createCoordinateAxisFile(dim,
                                           bildFileName=tmpFileName,
                                           sampling=sampling)
          f.write("run(session, 'open %s')\n" % tmpFileName)

          # Open atomstruct and color it by the bfactor (which is actually the DAQ score)
          cifDic = AtomicStructHandler().readLowLevel(cifFile)
          outFile = self.protocol._getExtraPath('chimeraAttribute_{}.cif'.format(attrName))
          if getStructureRecipient(cifDic, attrName=attrName) in ['atoms']:
              f.write("run(session, 'open %s')\n" % replaceOcuppancyWithAttribute(cifFile, attrName, outFile))
              attrColorName = 'occupancy'
              
          elif getStructureRecipient(cifDic, attrName=attrName) in ['residues'] and \
                  getCifKeyName(cifDic, 'asym') and getCifKeyName(cifDic, 'seq'):
              f.write("run(session, 'open %s')\n" % replaceOcuppancyWithAttribute(cifFile, attrName, outFile))
              attrColorName = 'occupancy'
          else:
              f.write("run(session, 'open %s')\n" % cifFile)
              defAttrFile = self.createAttributesFile(cifFile, strId)
              f.write("run(session, 'open %s')\n" % defAttrFile)
              attrColorName = attrName

          stepColors, colorList = self.getColors()
          scolorStr = ''
          for step, color in zip(stepColors, colorList):
            scolorStr += '%s,%s:' % (step, color)
          scolorStr = scolorStr[:-1]

          average = ''
          if getStructureRecipient(cifDic, attrName=attrName) == 'atoms' and self.residuesAverages:
              average = 'average residues'

          f.write("run(session, 'color byattribute {} palette {} {}')\n".
                  format(attrColorName, scolorStr, average))
          f.write(generateColorLegend(stepColors, colorList))
          f.write("run(session, 'view')\n")

          f.close()
          view = ChimeraView(chimScript)
          return [view]

      else:
          return [self.warnMessage(CHIMERA_ERROR, 'Chimera not found')]

    def _showHistogram(self, paramName=None):
      attrname = self.getEnumText('attrName')
      cifFile = self.getAtomStructObject().getFileName()
      cifDic = AtomicStructHandler().readLowLevel(cifFile)
      #TODO: admit non float attributes
      names, values = np.array(cifDic[NAME]), np.array(cifDic[VALUE])
      recipient = getStructureRecipient(cifDic, attrName=attrname)
      attrValues = values[names == attrname]
      attrValues = list(map(float, attrValues))

      self.plotter = EmPlotter(x=1, y=1, windowTitle=attrname)
      a = self.plotter.createSubPlot("{} counts".format(attrname), attrname, "{} count".format(recipient))
      low, high = self.getValuesRange()
      a.set_xlim([low, high])

      n = 5
      mult = 10 ** n
      stepSize = int(round((high-low) / self.intervals.get(), n) * mult)
      bins = [i / mult for i in range(int(low * mult), int(high * mult), stepSize)]
      _, _, bars = a.hist(attrValues, bins=bins, linewidth=1, label="Map", rwidth=0.9)

      colorSteps, colorList = self.getColors()
      colorList.reverse()
      for bar, colorS, color in zip(bars, colorSteps, colorList):
          bar.set_facecolor(color)

      a.grid(True)
      return [self.plotter]

    ###################### UTILS #########################
    def getValuesRange(self):
        if self.lowest.get() != self.highest.get():
            return self.lowest.get(), self.highest.get()
        else:
            attrname = self.getEnumText('attrName')
            cifDic = AtomicStructHandler().readLowLevel(self.getAtomStructObject().getFileName())
            names, values = np.array(cifDic[NAME]), np.array(cifDic[VALUE])
            recipient = cifDic[RECIP][0]
            attrValues = values[names == attrname]
            attrValues = list(map(float, attrValues))
            return min(attrValues), max(attrValues)

    def getColors(self):
      low, high = self.getValuesRange()
      stepColors = splitRange(high, low, splitNum=self.intervals.get())
      colorList = plotter.getHexColorList(len(stepColors), self.colorMap.get())
      return stepColors, colorList

    def createAttributesFile(self, cifFile, chiEleId):
        cifDic = AtomicStructHandler().readLowLevel(cifFile)

        attrName = self.getEnumText('attrName')
        defAttrStr = 'attribute: {}\n'.format(self.getEnumText('attrName'))
        first = True
        for name, recip, spec, value in zip(cifDic[NAME], cifDic[RECIP],
                                            cifDic[SPEC], cifDic[VALUE]):
            if name == attrName:
                if first:
                    defAttrStr += 'recipient: {}\n'.format(recip)
                    first = False
                defAttrStr += '\t#{}/{}\t{}\n'.format(chiEleId, spec, value)

        defattrFile = self.protocol._getExtraPath('{}.defattr'.format(attrName))
        with open(defattrFile, 'w') as f:
            f.write(defAttrStr)
        return defattrFile


def chimeraInstalled():
  return Chimera.getHome() and os.path.exists(Chimera.getProgram())

def getCifKeyName(cifDic, keyBase):
    base = '_atom_site.{}' + '_{}_id'.format(keyBase)
    options = [base.format('pdbx_auth'), base.format('auth'), base.format('label')]
    for name in options:
      if name in cifDic:
        return name

def replaceOcuppancyWithAttribute(cifFile, attrName, outFile=None):
    '''Instead of defining the atribute in a defattr file, switch it with the occupancy and color by it.
    It notably speeds up chimera colouring'''
    cifDic = AtomicStructHandler().readLowLevel(cifFile)
    if not outFile:
        outDir = os.path.dirname(cifFile)
        outFile = os.path.join(outDir, 'chimeraAttribute_{}.cif'.format(attrName))

    if not os.path.exists(outFile):
        names, values, specs = np.array(cifDic[NAME]), np.array(cifDic[VALUE]), np.array(cifDic[SPEC])
        recipient = getStructureRecipient(cifDic, attrName)
        attrValues, attrSpecs = values[names == attrName], specs[names == attrName]
        if recipient == 'atoms':
            cifDic['_atom_site.occupancy'] = attrValues
        elif recipient == 'residues':
            atomValues = []
            resDic = makeResidueValuesDic(cifDic, attrName)

            chainsStr, resStr = getCifKeyName(cifDic, 'asym'), getCifKeyName(cifDic, 'seq')
            for resChain, resNumber in zip(cifDic[chainsStr], cifDic[resStr]):
                resKey = '{}:{}'.format(resChain, resNumber)
                if resKey in resDic:
                    #HOH atoms may be ignored and not appear in the cif
                    lastVal = resDic[resKey]
                    atomValues.append(lastVal)
                else:
                    atomValues.append(lastVal)

            cifDic['_atom_site.occupancy'] = atomValues

        AtomicStructHandler()._writeLowLevel(outFile, cifDic)
    return outFile

def getResidueAverage(atomValues, atomSpec):
    resDic = {}
    for atVal, atSpe in zip(atomValues, atomSpec):
        resSpe = atSpe.split('@')[0]
        if resSpe in resDic:
            resDic[resSpe].append(atVal)
        else:
            resDic[resSpe] = [atVal]

    resValues = []
    for resSpe in resDic:
        resValues.append(np.mean(list(map(float, resDic[resSpe]))))
    return resValues, list(resDic.keys())

def getResiduePositions(attrSpecs, attrValues, chainName):
    attrPositions, attrChainValues = [], []
    for spe, val in zip(attrSpecs, attrValues):
        chain, position = spe.split(':')
        if chain == chainName:
            attrPositions.append(position)
            attrChainValues.append(val)
    return attrPositions, attrChainValues

def getStructureRecipient(cifDic, attrName):
    '''Returns a list with the names of the attributes of the output object'''
    recipDic = {}
    for name, recip in zip(cifDic[NAME], cifDic[RECIP]):
        recipDic[name] = recip
    return recipDic[attrName]

def makeResidueValuesDic(cifDic, attrName):
    resDic = {}
    names, values, specs = np.array(cifDic[NAME]), np.array(cifDic[VALUE]), np.array(cifDic[SPEC])
    attrValues, attrSpecs = values[names == attrName], specs[names == attrName]
    for spec, val in zip(attrSpecs, attrValues):
        resDic[spec] = val
    return resDic



#!/usr/bin/env python

# ***************************************************************************
# * Authors:     Roberto Marabini (roberto@cnb.csic.es)
# *              Jordi Burguet Castell (jburguet@cnb.csic.es)
# *
# * Unidad de Bioinformatica of Centro Nacional de Biotecnologia, CSIC
# *
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation; either version 3 of the License, or
# * (at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program; if not, write to the Free Software
# * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
# * 02111-1307  USA
# *
# *  All comments concerning this program package may be sent to the
# *  e-mail address 'scipion@cnb.csic.es'
# *
# **************************************************************************


import logging
logger = logging.getLogger(__name__)
import random
import unittest

from pwem.protocols import ProtUserSubSet
from pyworkflow.tests import DataSet

try:
    from itertools import izip
except ImportError:
    izip = zip

import pyworkflow.tests as pwtests
import pyworkflow.utils as pwutils

import pwem.protocols as emprot
import pwem.objects as emobj

# Used by Roberto's test, where he creates the particles "by hand"
from pwem.objects.data import Particle, SetOfParticles, Acquisition, CTFModel
from pyworkflow.utils.utils import prettyDict
from pyworkflow.object import Float


class TestSets(pwtests.BaseTest):
    """Run different tests related to the set operations."""

    @classmethod
    def setUpClass(cls):
        """Prepare the data that we will use later on."""

        logger.info(pwutils.greenStr(" Set Up - Collect data ".center(75, '-')))

        pwtests.setupTestProject(cls)  # defined in BaseTest, creates cls.proj

        cls.dataset_xmipp = pwtests.DataSet.getDataSet('xmipp_tutorial')
        cls.dataset_mda = pwtests.DataSet.getDataSet('mda')
        cls.dataset_ribo = pwtests.DataSet.getDataSet('ribo_movies')
        cls.datasetRelion = pwtests.DataSet.getDataSet('relion_tutorial')

        #
        # Imports
        #
        new = cls.proj.newProtocol  # short notation
        launch = cls.proj.launchProtocol
        # Micrographs
        # NOTE: This dataset has 3 mic with heterogeneous dimensions!! But so
        # far is not failing, should it?
        logger.info(pwutils.magentaStr("==> Importing data - micrographs"))
        p_imp_micros = new(emprot.ProtImportMicrographs,
                           filesPath=cls.dataset_xmipp.getFile('allMics'),
                           samplingRate=1.237, voltage=300)
        launch(p_imp_micros, wait=True)
        cls.micros = p_imp_micros.outputMicrographs

        # Micrographs SMALL - This is a mic with different dimensions
        logger.info(pwutils.magentaStr("==> Importing data - micrographs SMALL"))
        p_imp_micros = new(emprot.ProtImportMicrographs,
                           filesPath=cls.dataset_xmipp.getFile('mic3'),
                           samplingRate=1.237, voltage=300)
        launch(p_imp_micros, wait=True)
        cls.microsSmall = p_imp_micros.outputMicrographs

        # Volumes
        logger.info(pwutils.magentaStr("==> Importing data - volumes"))
        p_imp_volumes = new(emprot.ProtImportVolumes,
                            filesPath=cls.dataset_xmipp.getFile('volumes'),
                            samplingRate=9.896)
        launch(p_imp_volumes, wait=True)
        cls.vols = p_imp_volumes.outputVolumes

        # Movies
        logger.info(pwutils.magentaStr("==> Importing data - movies"))
        p_imp_movies = new(emprot.ProtImportMovies,
                           filesPath=cls.dataset_ribo.getFile('movies'),
                           samplingRate=2.37, magnification=59000,
                           voltage=300, sphericalAberration=2.0,
                           gainFile=cls.dataset_ribo.getFile('volume'),
                           darkFile=cls.dataset_ribo.getFile('volume'))
        launch(p_imp_movies, wait=True)
        cls.movies = p_imp_movies.outputMovies

        # Particles
        logger.info(pwutils.magentaStr("==> Importing data - particles"))
        p_imp_particles = new(emprot.ProtImportParticles,
                              filesPath=cls.dataset_mda.getFile('particles'),
                              samplingRate=3.5)
        launch(p_imp_particles, wait=True)
        cls.particles = p_imp_particles.outputParticles

        # Particles with micId
        logger.info(pwutils.magentaStr("==> Importing data - particles with micId"))
        relionFile = 'import/case2/relion_it015_data.star'
        pImpPartMicId = new(emprot.ProtImportParticles,
                            objLabel='from relion (auto-refine 3d)',
                            importFrom=emprot.ProtImportParticles.IMPORT_FROM_RELION,
                            starFile=cls.datasetRelion.getFile(relionFile),
                            magnification=10000,
                            samplingRate=7.08,
                            haveDataBeenPhaseFlipped=True)
        launch(pImpPartMicId, wait=True)
        cls.partMicId = pImpPartMicId.outputParticles
        cls.micsMicId = pImpPartMicId.outputMicrographs

        # Coordinates  -  Oh, I don't know of any example of coord. import :(

    #
    # Helper functions
    #
    def split(self, em_set, n, randomize):
        """Return a run split protocol over input set em_set."""

        p_split = self.proj.newProtocol(emprot.ProtSplitSet, numberOfSets=n)
        p_split.inputSet.set(em_set)
        p_split.randomize.set(randomize)
        self.proj.launchProtocol(p_split, wait=True)
        return p_split

    @staticmethod
    def outputs(p):
        """Iterator over all the elements in the outputs of protocol p."""
        for key, output in p.iterOutputAttributes():
            yield output

    #
    # The tests themselves.
    #
    def testSplit(self):
        """Test that the split operation works as expected."""

        logger.info(pwutils.greenStr(" Test Split ".center(75, '-')))

        def check(set0, n=2, randomize=False):
            # Simple checks on split sets from set0.
            logger.info(pwutils.magentaStr("==> Check split of %s" % type(set0).__name__))
            unsplit_set = [x.strId() for x in set0]
            p_split = self.split(set0, n=n, randomize=randomize)
            # Are all output elements of the protocol in the original set?
            for em_set in self.outputs(p_split):
                for elem in em_set:
                    self.assertTrue(elem.strId() in unsplit_set)
            # Number of elements of all split sets equal to original number?
            self.assertEqual(sum(len(x) for x in self.outputs(p_split)),
                             len(set0))

        check(self.micros)
        check(self.micros, randomize=True)
        check(self.vols)
        check(self.movies)
        check(self.particles)
        check(self.particles, n=4)

    def testSubsetByParams(self):
        """Test that the subset operation using parameters works as expected."""

        logger.info(pwutils.greenStr(" Test Subset by params".center(75, '-')))

        # Launch random subset
        p_subset = self.newProtocol(emprot.ProtSubSet)
        p_subset.setObjLabel('Random subset')
        p_subset.inputFullSet.set(self.particles)
        p_subset.chooseAtRandom.set(True)
        p_subset.nElements.set(10)
        self.launchProtocol(p_subset)

        self.assertSetSize(p_subset.outputParticles, 10)

        # Launch subset by ids
        p_subset = self.newProtocol(emprot.ProtSubSet)
        p_subset.setObjLabel('Subset by ids')
        p_subset.inputFullSet.set(self.particles)
        p_subset.selectIds.set(True)
        p_subset.range.set("1-4, 8, 1000") # Last one 1000 should be skipped as is missing in the set.
        self.launchProtocol(p_subset)

        self.assertSetSize(p_subset.outputParticles, 5)
        self.assertIsNotNone(p_subset.outputParticles[8], "Subset by id did not picked item 8.")

    def testSubsetByRange(self):
        logger.info(pwutils.greenStr(" Test Subset by Range"))

        particles = [p.clone() for p in self.particles]

        def _equal(part1, part2):
            return part1.getObjDict() == part2.getObjDict()

        # Launch random subset
        ps1 = self.newProtocol(emprot.ProtSubSet)
        ps1.inputFullSet.set(self.particles)
        ps1.setObjLabel('Range 1')
        ps1.selectIds.set(True)
        ps1.range.set('1-10')
        self.launchProtocol(ps1)
        self.assertSetSize(ps1.outputParticles, 10)
        self.assertTrue(_equal(particles[0], ps1.outputParticles.getFirstItem()))

        ps2 = self.newProtocol(emprot.ProtSubSet)
        ps2.inputFullSet.set(self.particles)
        ps2.setObjLabel('Range 2')
        ps2.selectIds.set(True)
        ps2.range.set('11-20')
        self.launchProtocol(ps2)
        self.assertSetSize(ps2.outputParticles, 10)
        self.assertTrue(_equal(particles[10], ps2.outputParticles.getFirstItem()))

    def testSubsetIntersection(self):
        """Test that the subset operation works as expected."""

        logger.info(pwutils.greenStr(" Test Subset ".center(75, '-')))

        def check(set0, n1=2, n2=2):
            """Simple checks on subsets, coming from split sets of set0."""
            logger.info(pwutils.magentaStr("==> Check subset of %s" % type(set0).__name__))
            p_split1 = self.split(set0, n=n1, randomize=True)
            p_split2 = self.split(set0, n=n2, randomize=True)

            setFull = random.choice(list(self.outputs(p_split1)))
            setSub = random.choice(list(self.outputs(p_split2)))

            label = '%s - %s,%s ' % (set0.getClassName(), n1, n2)
            # Launch intersection subset
            p_subset = self.newProtocol(emprot.ProtSubSet)
            p_subset.setObjLabel(label + 'intersection')
            p_subset.inputFullSet.set(setFull)
            p_subset.inputSubSet.set(setSub)
            self.launchProtocol(p_subset)

            # Launch difference subset
            p_subset_diff = self.proj.copyProtocol(p_subset)
            p_subset_diff.setOperation.set(p_subset_diff.SET_DIFFERENCE)
            p_subset_diff.setObjLabel(label + 'difference')
            self.launchProtocol(p_subset_diff)

            setFullIds = setFull.getIdSet()
            setSubIds = setSub.getIdSet()
            n = len(setFull)

            # Check intersection
            outputs = [o for o in self.outputs(p_subset)]
            n1 = 0
            if outputs:
                output = outputs[0]

                # Check properties
                self.assertTrue(set0.equalAttributes(output,
                                                     ignore=['_mapperPath', '_size'],
                                                     verbose=True),
                                "Intersection subset attributes are wrong")

                n1 = len(output)
                for elem in output:
                    self.assertTrue(elem.getObjId() in setFullIds)
                    self.assertTrue(elem.getObjId() in setSubIds,
                                    'object id %s not in set: %s'
                                    % (elem.getObjId(), setSubIds))

            # Check difference
            outputs = [o for o in self.outputs(p_subset_diff)]
            n2 = 0
            if outputs:
                output_diff = outputs[0]
                # Check properties
                self.assertTrue(set0.equalAttributes(output_diff,
                                                     ignore=['_mapperPath', '_size'],
                                                     verbose=True),
                                "In subset attributes are wrong")

                n2 = len(output_diff)
                for elem in output_diff:
                    self.assertTrue(elem.getObjId() in setFullIds)
                    self.assertTrue(elem.getObjId() not in setSubIds)

            self.assertTrue(n >= n1)
            self.assertTrue(n >= n2)
            self.assertEqual(n, n1 + n2)

        check(self.movies)
        check(self.particles)
        check(self.particles, n1=3, n2=5)

    def testSubsetByMic(self):
        """Test that the subset by Mic operation works as expected."""
        logger.info( pwutils.greenStr(" Test Subset by Mic".center(75, '-')))
        "Simple checks on subsets, coming from split sets of setMics."
        logger.info(pwutils.magentaStr("==> Check subset of %s by %s"
                                 % (type(self.partMicId).__name__,
                                    type(self.micsMicId).__name__)))

        # launch the protocol for a certain mics input
        def launchSubsetByMic(micsSubset):
            pSubsetbyMic = self.newProtocol(emprot.ProtSubSetByMic)
            pSubsetbyMic.inputParticles.set(self.partMicId)
            pSubsetbyMic.inputMicrographs.set(micsSubset)
            self.launchProtocol(pSubsetbyMic)
            return pSubsetbyMic.outputParticles

        # Check if the Output is generated, the subset size is correct and
        #  the micId of the particle with certain partId is correct.
        def checkAsserts(setParts, size, partId, micId):
            self.assertIsNotNone(setParts, "Output SetOfParticles"
                                           " were not created.")
            self.assertEqual(setParts.getSize(), size,
                             "The number of created particles is incorrect.")
            p = setParts[partId]
            self.assertEqual(p.getMicId(), micId)

        # Whole set of micrographs
        setMics = self.micsMicId
        # Create a subsets of Mics to apply the protocol
        pSplit = self.split(setMics, n=2, randomize=False)
        setMics2 = pSplit.outputMicrographs02
        # Create a subset of a single micrograph to apply the protocol
        pSplit = self.split(setMics, n=20, randomize=False)
        setMics3 = pSplit.outputMicrographs03

        # Launch subset by mics protocol with the whole set of Mics
        partByMic1 = launchSubsetByMic(setMics)
        # Launch subset by mics protocol with a subset of Mics
        partByMic2 = launchSubsetByMic(setMics2)
        # Launch subset by mics protocol with a single SetOfMics
        partByMic3 = launchSubsetByMic(setMics3)

        # Assertions for the three sets
        checkAsserts(partByMic1, self.partMicId.getSize(), 1885, 7)
        checkAsserts(partByMic2, 2638, 4330, 16)
        checkAsserts(partByMic3, 270, 725, 3)

    def testSubsetByCoord(self):
        """Test that the subset by Coord operation works as expected."""
        logger.info(pwutils.greenStr(" Test Subset by Coord".center(75, '-')))

        p_extract_coordinates = self.newProtocol(emprot.ProtExtractCoords)
        p_extract_coordinates.inputParticles.set(self.partMicId)
        p_extract_coordinates.inputMicrographs.set(self.micsMicId)
        self.launchProtocol(p_extract_coordinates)

        p_subset_by_coords = self.newProtocol(emprot.ProtSubSetByCoord)
        p_subset_by_coords.inputParticles.set(self.partMicId)
        p_subset_by_coords.inputCoordinates.set(p_extract_coordinates.outputCoordinates)
        self.launchProtocol(p_subset_by_coords)
        self.assertIsNotNone(p_subset_by_coords.outputParticles, "Output SetOfParticles were not created.")
        self.assertEqual(p_subset_by_coords.outputParticles.getSize(), 5236,
                         "The number of created particles is incorrect.")

    def testMerge(self):
        """Test that the union operation works as expected."""

        logger.info(pwutils.greenStr(" Test Merge ".center(75, '-')))

        def check(set0):
            # Simple checks on merge, coming from many split sets of set0.
            logger.info(pwutils.magentaStr("==> Check merge of %s" % type(set0).__name__))
            p_union = self.proj.newProtocol(emprot.ProtUnionSet)

            setsIds = []
            for i in range(random.randint(1, 5)):
                n = random.randint(1, len(set0) // 2)
                p_split = self.split(set0, n=n, randomize=True)
                setRandom = random.choice(list(self.outputs(p_split)))
                setsIds.append([x.strId() for x in setRandom])
                p_union.inputSets.append(setRandom)
            self.proj.launchProtocol(p_union, wait=True)

            output = next(self.outputs(p_union))  # first (and only!) output
            self.assertEqual(len(output), sum(len(x) for x in setsIds))
            # We might be able to do more interesting tests, using the
            # collected setsIds.

        check(self.micros)
        check(self.vols)
        check(self.movies)
        check(self.particles)

    def testMergeAlternateColumn(self):
        """Test that the union operation works as expected.
        Even if the order of the columns do not match.
        That is, M1(a,b,c) U M2(a,c,b)"""
        # Create two sets of particles
        inFileNameMetadata1 = self.proj.getTmpPath('particles1.sqlite')
        inFileNameMetadata2 = self.proj.getTmpPath('particles2.sqlite')
        imgSet1 = SetOfParticles(filename=inFileNameMetadata1)
        imgSet2 = SetOfParticles(filename=inFileNameMetadata2)

        inFileNameData = self.proj.getTmpPath('particles.stk')
        img1 = Particle()
        img2 = Particle()
        attrb1 = [11, 12, 13, 14]
        attrb2 = [21, 22, 23, 24]
        counter = 0

        for i in range(1, 3):
            img1.cleanObjId()
            img1.setLocation(i, inFileNameData)
            img1.setMicId(i % 3)
            img1.setClassId(i % 5)
            img1.setSamplingRate(1.)
            img1._attrb1 = Float(attrb1[counter])
            img1._attrb2 = Float(attrb2[counter])
            imgSet1.append(img1)
            counter += 1

        for i in range(1, 3):
            img2.cleanObjId()
            img2.setLocation(i, inFileNameData)
            img2.setClassId(i % 5)
            img2.setMicId(i % 3)
            img2.setSamplingRate(2.)
            img2._attrb1 = Float(attrb1[counter])
            img2._attrb2 = Float(attrb2[counter])
            imgSet2.append(img2)
            counter += 1

        imgSet1.write()
        imgSet2.write()

        # import them
        protImport1 = self.newProtocol(emprot.ProtImportParticles,
                                       objLabel='import set1',
                                       importFrom=emprot.ProtImportParticles.IMPORT_FROM_SCIPION,
                                       sqliteFile=inFileNameMetadata1,
                                       magnification=10000,
                                       samplingRate=7.08,
                                       haveDataBeenPhaseFlipped=True)
        self.launchProtocol(protImport1)

        protImport2 = self.newProtocol(emprot.ProtImportParticles,
                                       objLabel='import set2',
                                       importFrom=emprot.ProtImportParticles.IMPORT_FROM_SCIPION,
                                       sqliteFile=inFileNameMetadata2,
                                       magnification=10000,
                                       samplingRate=7.08,
                                       haveDataBeenPhaseFlipped=True)
        self.launchProtocol(protImport2)

        # create merge protocol
        p_union = self.newProtocol(emprot.ProtUnionSet,
                                   objLabel='join diff column order',
                                   ignoreExtraAttributes=True)
        p_union.inputSets.append(protImport1.outputParticles)
        p_union.inputSets.append(protImport2.outputParticles)
        self.proj.launchProtocol(p_union, wait=True)
        # assert
        counter = 0
        for img in p_union.outputSet:
            self.assertAlmostEqual(attrb1[counter], img._attrb1, 4)
            self.assertAlmostEqual(attrb2[counter], img._attrb2, 4)
            counter += 1

    def testMergeDifferentAttrs(self):
        """ Test merge from subsets with different attributes.
        That is, M1(a,b,c) U M2(a,b,c,d)"""

        # create two set of particles
        inFileNameMetadata1 = self.proj.getTmpPath('particles11.sqlite')
        inFileNameMetadata2 = self.proj.getTmpPath('particles22.sqlite')
        imgSet1 = SetOfParticles(filename=inFileNameMetadata1)
        imgSet2 = SetOfParticles(filename=inFileNameMetadata2)

        inFileNameData = self.proj.getTmpPath('particles.stk')
        # Start ids 4
        img1 = Particle(objId=4)
        img2 = Particle(objId=4)
        attrb1 = [11, 12, 13, 14]
        attrb2 = [21, 22, 23, 24]
        attrb3 = [31, 32]
        counter = 0
        # Test the join handles different attributes at a second level
        ctf1 = CTFModel(defocusU=1000, defocusV=1000, defocusAngle=0)
        ctf2 = CTFModel(defocusU=2000, defocusV=2000, defocusAngle=0)
        ctf2._myOwnQuality = Float(1.)
        img1.setCTF(ctf1)
        img2.setCTF(ctf2)

        for i in range(1, 3):
            # Increment Id
            img1.setObjId(img1.getObjId() + 1)
            img1.setLocation(i, inFileNameData)
            img1.setMicId(i % 3)
            img1.setClassId(i % 5)
            img1.setSamplingRate(1.)
            img1._attrb1 = Float(attrb1[counter])
            img1._attrb2 = Float(attrb2[counter])
            img1._attrb3 = Float(attrb3[counter])
            imgSet1.append(img1)
            counter += 1

        for i in range(1, 3):
            # Increment Id
            img2.setObjId(img2.getObjId() + 1)
            img2.setLocation(i, inFileNameData)
            img2.setClassId(i % 5)
            img2.setMicId(i % 3)
            img2.setSamplingRate(2.)
            img2._attrb1 = Float(attrb1[counter])
            img2._attrb2 = Float(attrb2[counter])
            imgSet2.append(img2)
            counter += 1

        imgSet1.write()
        imgSet2.write()

        # import them
        protImport1 = self.newProtocol(emprot.ProtImportParticles,
                                       objLabel='import set1',
                                       importFrom=emprot.ProtImportParticles.IMPORT_FROM_SCIPION,
                                       sqliteFile=inFileNameMetadata1,
                                       magnification=10000,
                                       samplingRate=7.08,
                                       haveDataBeenPhaseFlipped=True
                                       )
        self.launchProtocol(protImport1)

        protImport2 = self.newProtocol(emprot.ProtImportParticles,
                                       objLabel='import set2',
                                       importFrom=emprot.ProtImportParticles.IMPORT_FROM_SCIPION,
                                       sqliteFile=inFileNameMetadata2,
                                       magnification=10000,
                                       samplingRate=7.08,
                                       haveDataBeenPhaseFlipped=True
                                       )
        self.launchProtocol(protImport2)

        # create merge protocol
        p_union = self.newProtocol(emprot.ProtUnionSet,
                                   objLabel='join different attrs',
                                   ignoreExtraAttributes=True)
        p_union.inputSets.append(protImport1.outputParticles)
        p_union.inputSets.append(protImport2.outputParticles)
        self.proj.launchProtocol(p_union, wait=True)

        counter = 0

        for img in p_union.outputSet:
            self.assertAlmostEqual(attrb1[counter], img._attrb1, 4)
            self.assertAlmostEqual(attrb2[counter], img._attrb2, 4)
            self.assertFalse(hasattr(img, '_attrb3'),
                             "join should not have attrb3")
            self.assertTrue(hasattr(img, '_attrb2'),
                            "join should have attrb2")
            ctf = img.getCTF()
            self.assertIsNotNone(ctf, "Image should have CTF after join")
            self.assertFalse(hasattr(ctf, '_myOwnQuality'),
                             "CTF should not have non common attributes")

            # Assert ids
            self.assertEqual(counter + 5, img.getObjId(),
                             "Object id's not kept.")
            counter += 1

    def testOrderBy(self):
        """ create set of particles and orderby a given attribute
        """
        # This function was written by Roberto. It does things
        # differently, so let's keep it for reference.

        # create set of particles

        inFileNameMetadata = self.proj.getTmpPath('particlesOrderBy.sqlite')
        inFileNameData = self.proj.getTmpPath('particlesOrderBy.stk')

        imgSet = SetOfParticles(filename=inFileNameMetadata)
        imgSet.setSamplingRate(1.5)
        acq = Acquisition()
        acq.setAmplitudeContrast(0.1)
        acq.setMagnification(10000)
        acq.setVoltage(200)
        acq.setSphericalAberration(2.0)

        imgSet.setAcquisition(acq)
        img = Particle()

        for i in range(1, 10):
            img.setLocation(i, inFileNameData)
            img.setMicId(i % 3)
            img.setClassId(i % 5)
            imgSet.append(img)
            img.cleanObjId()

        imgSet.write()
        # now import the dataset
        prot1 = self.newProtocol(emprot.ProtImportParticles,
                                 importFrom=emprot.ProtImportParticles.IMPORT_FROM_SCIPION,
                                 sqliteFile=inFileNameMetadata,
                                 magnification=10000,
                                 samplingRate=1.5
                                 )
        prot1.setObjLabel('from sqlite (test-sets)')
        self.launchProtocol(prot1)

        if prot1.outputParticles is None:
            raise Exception(
                'Import of images: %s, failed. outputParticles is None.'
                % inFileNameMetadata)

        protSplitSet = self.newProtocol(emprot.ProtSplitSet,
                                        inputSet=prot1.outputParticles,
                                        numberOfSets=2,
                                        randomize=True)
        self.launchProtocol(protSplitSet)

        inputSets = [protSplitSet.outputParticles01,
                     protSplitSet.outputParticles02]
        outputSet = SetOfParticles(filename=self.proj.getTmpPath('gold.sqlite'))
        for itemSet in inputSets:
            for obj in itemSet:
                outputSet.append(obj)

        for item1, item2 in izip(imgSet, outputSet):
            if not item1.equalAttributes(item2):
                logger.info("Items differ:")
                prettyDict(item1.getObjDict())
                prettyDict(item2.getObjDict())
            self.assertTrue(item1.equalAttributes(item2), )

    def testEmptiness(self):

        self.assertSetSize(self.particles)
        self.assertSetSize(self.particles, 76)

    def testJoinValidation(self):

        # Create a merge protocol
        p_union = self.newProtocol(emprot.ProtUnionSet,
                                   objLabel='invalid join',
                                   ignoreExtraAttributes=True)
        p_union.inputSets.append(self.microsSmall)
        p_union.inputSets.append(self.micros)

        with self.assertRaises(Exception):
            self.launchProtocol(p_union)


class TestUserSubSet(pwtests.BaseTest):
    @classmethod
    def setUpClass(cls):
        cls.dataset = DataSet.getDataSet('model')
        cls.selectionFn = cls.dataset.getFile('classesSelection')
        pwtests.setupTestProject(cls)

    def test_2DClasses(self):
        """ Load an existing SetOfClasses and test basic properties
        such us: _mapperPath, iteration and others.
        """
        emProt = self.newProtocol(emprot.ProtImportParticles)
        classes2DSet = emobj.SetOfClasses2D(filename=self.selectionFn)
        emProt._defineOutputs(outputClasses=classes2DSet)
        emProt.setFinished()
        emProt._store()

        batchProt = self.newProtocol(ProtUserSubSet,
                                     inputObject=classes2DSet,
                                     sqliteFile=self.selectionFn + ',',
                                     outputClassName='SetOfClasses2D')
        self.launchProtocol(batchProt)


if __name__ == '__main__':
    unittest.main()



# **************************************************************************
# *
# * Authors:    Laura del Cano (ldelcano@cnb.csic.es)
# *
# * Unidad de  Bioinformatica of Centro Nacional de Biotecnologia , CSIC
# *
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation; either version 3 of the License, or
# * (at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program; if not, write to the Free Software
# * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
# * 02111-1307  USA
# *
# *  All comments concerning this program package may be sent to the
# *  e-mail address 'scipion@cnb.csic.es'
# *
# **************************************************************************

import numpy as np

import pyworkflow.tests as pwtests
from pwem import Domain
import pwem.protocols as emprot

from .test_workflow import TestWorkflow


class TestXmippWorkflow(TestWorkflow):

    @classmethod
    def setUpClass(cls):
        pwtests.setupTestProject(cls)
        cls.dataset = pwtests.DataSet.getDataSet('xmipp_tutorial')
        cls.allCrdsDir = cls.dataset.getFile('posAllDir')
        cls.micsFn = cls.dataset.getFile('allMics')
        cls.vol1 = cls.dataset.getFile('vol1')

    def testXmippWorkflow(self):
        # First, import a set of micrographs
        protImport = self.newProtocol(emprot.ProtImportMicrographs,
                                      filesPath=self.micsFn,
                                      samplingRate=1.237, voltage=300)
        self.launchProtocol(protImport)
        self.assertSetSize(protImport.outputMicrographs, 3)
        self.assertIsNotNone(protImport.outputMicrographs.getFileName(),
                             "There was a problem with the import")
        self.validateFiles('protImport', protImport)

        # Import a set of volumes
        print("Import Volume")
        protImportVol = self.newProtocol(emprot.ProtImportVolumes,
                                         filesPath=self.vol1,
                                         samplingRate=9.896)
        self.launchProtocol(protImportVol)
        self.assertIsNotNone(protImportVol.getFiles(),
                             "There was a problem with the import")
        #        self.validateFiles('protImportVol', protImportVol)

        # Perform a downsampling on the micrographs
        print("Downsampling...")
        xmippProtcols = Domain.importFromPlugin('xmipp3.protocols',
                                                doRaise=True)
        protDownsampling = self.newProtocol(xmippProtcols.XmippProtPreprocessMicrographs,
                                            doDownsample=True,
                                            downFactor=5,
                                            doCrop=False,
                                            runMode=1)
        protDownsampling.inputMicrographs.set(protImport.outputMicrographs)
        self.launchProtocol(protDownsampling)
        self.assertSetSize(protDownsampling.outputMicrographs, 3)
        self.assertIsNotNone(protDownsampling.outputMicrographs,
                             "There was a problem with the downsampling")
        self.validateFiles('protDownsampling', protDownsampling)

        print("Importing coordinates")
        protPP = self.newProtocol(emprot.ProtImportCoordinates,
                                  importFrom=emprot.ProtImportCoordinates.IMPORT_FROM_XMIPP,
                                  filesPath=self.allCrdsDir,
                                  filesPattern='*.pos', boxSize=110)

        protPP.inputMicrographs.set(protDownsampling.outputMicrographs)
        self.launchProtocol(protPP)
        self.protDict['protPicking'] = protPP
        self.assertSetSize(protPP.outputCoordinates, 143 + 138 + 122 + 110 + 138 + 122,
                           "There was a problem with the import of coordinates")

        # Now estimate CTF on the downsampled micrographs
        print("Performing CTF...")
        protCTF = self.newProtocol(xmippProtcols.XmippProtCTFMicrographs,
                                   numberOfThreads=4,
                                   minDefocus=2.2,
                                   maxDefocus=2.5)  # Defocus is in microns
        protCTF.inputMicrographs.set(protDownsampling.outputMicrographs)
        self.launchProtocol(protCTF)
        self.assertSetSize(protCTF.outputCTF, 3)
        self.assertIsNotNone(protCTF.outputCTF,
                             "There was a problem with the CTF estimation")
        # After CTF estimation, the output micrograph should have CTF info
        self.validateFiles('protCTF', protCTF)

        print("Run extract particles with other downsampling factor")
        protExtract = self.newProtocol(xmippProtcols.XmippProtExtractParticles,
                                       boxSize=64, downsampleType=emprot.OTHER,
                                       doFlip=True, downFactor=8,
                                       runMode=1, doInvert=True)
        protExtract.inputCoordinates.set(protPP.outputCoordinates)
        protExtract.ctfRelations.set(protCTF.outputCTF)
        protExtract.inputMicrographs.set(protImport.outputMicrographs)
        self.launchProtocol(protExtract)
        self.assertIsNotNone(protExtract.outputParticles,
                             "There was a problem with the extract particles")
        self.validateFiles('protExtract', protExtract)

        print("Run Extract Coordinates without applying shifts")
        protExtractCoords = self.newProtocol(emprot.ProtExtractCoords)
        protExtractCoords.inputParticles.set(protExtract.outputParticles)
        protExtractCoords.inputMicrographs.set(protImport.outputMicrographs)
        self.launchProtocol(protExtractCoords)
        # The size of the set of coordinates must be the same as the input set of particles
        self.assertSetSize(protExtractCoords.outputCoordinates,
                           size=protExtract.outputParticles.getSize(),
                           msg="There was a problem with the coordinates extraction")
        # Check if the scaling factor is being calculated and applied correctly
        scale = protExtract.outputParticles.getSamplingRate() / protImport.outputMicrographs.getSamplingRate()
        inParticleCoord = protExtract.outputParticles.getFirstItem().getCoordinate()
        x, y = inParticleCoord.getPosition()
        np.testing.assert_allclose(protExtractCoords.outputCoordinates.getFirstItem().getPosition(),
                                   (int(x * scale), int(y * scale)), rtol=0.5)

        print("Run Screen Particles")
        protScreen = self.newProtocol(xmippProtcols.XmippProtScreenParticles,
                                      autoParRejection=xmippProtcols.XmippProtScreenParticles.REJ_MAXZSCORE,
                                      maxZscore=3.0)
        protScreen.inputParticles.set(protExtract.outputParticles)
        self.launchProtocol(protScreen)
        self.assertSetSize(protScreen.outputParticles,
                           msg="There was a problem with Screen Particles")

        print("Run ML2D")
        protML2D = self.newProtocol(xmippProtcols.XmippProtML2D,
                                    numberOfClasses=1,
                                    maxIters=4,
                                    doMlf=True,
                                    numberOfMpi=2,
                                    numberOfThreads=1)
        protML2D.inputParticles.set(protScreen.outputParticles)
        self.launchProtocol(protML2D)
        self.assertIsNotNone(protML2D.outputClasses,
                             "There was a problem with ML2D")
        self.validateFiles('protML2D', protML2D)

        print("Run CL2D")
        protCL2D = self.newProtocol(xmippProtcols.XmippProtCL2D,
                                    numberOfClasses=2,
                                    numberOfInitialClasses=1,
                                    numberOfIterations=4, numberOfMpi=2)
        protCL2D.inputParticles.set(protExtract.outputParticles)
        self.launchProtocol(protCL2D)
        self.assertIsNotNone(protCL2D.outputClasses, "There was a problem with CL2D")
        self.validateFiles('protCL2D', protCL2D)

        print("Run Only Align2d")
        protOnlyAlign = self.newProtocol(xmippProtcols.XmippProtCL2DAlign,
                                         maximumShift=5, numberOfIterations=2,
                                         numberOfMpi=2, numberOfThreads=1, useReferenceImage=False)
        protOnlyAlign.inputParticles.set(protExtract.outputParticles)
        self.launchProtocol(protOnlyAlign)
        self.assertIsNotNone(protOnlyAlign.outputParticles, "There was a problem with Only align2d")
        self.validateFiles('protOnlyAlign', protOnlyAlign)

        print("Run Extract Coordinates applying shifts")
        protExtractCoordsShifts = self.newProtocol(emprot.ProtExtractCoords, applyShifts=True)
        protExtractCoordsShifts.setObjLabel('Extract Coordinates applying shifts')
        inputParticles = protOnlyAlign.outputParticles
        inputMics = protDownsampling.outputMicrographs
        protExtractCoordsShifts.inputParticles.set(inputParticles)
        protExtractCoordsShifts.inputMicrographs.set(inputMics)
        self.launchProtocol(protExtractCoordsShifts)
        # The size of the set of coordinates must be the same as the input set of particles
        outputParticles = protExtractCoordsShifts.outputCoordinates
        self.assertSetSize(outputParticles,
                           size=protExtract.outputParticles.getSize(),
                           msg="There was a problem with the coordinates extraction")
        # Check if the scaling factor is being calculated and applied correctly
        scale = inputParticles.getSamplingRate() / protDownsampling.outputMicrographs.getSamplingRate()
        inParticleCoord = inputParticles.getFirstItem().getCoordinate()
        shifts = protExtractCoordsShifts.getShifts(inputParticles.getFirstItem().getTransform(),
                                                   inputParticles.getAlignment())
        x, y = inParticleCoord.getPosition()
        xCoor, yCoor = x - int(shifts[0]), y - int(shifts[1])
        np.testing.assert_allclose(protExtractCoordsShifts.outputCoordinates.getFirstItem().getPosition(),
                                   (int(xCoor * scale), int(yCoor * scale)), rtol=0.5)

        print("Run kerdensom")
        ProtKerdensom = self.newProtocol(xmippProtcols.XmippProtKerdensom,
                                         useMask=False, SomXdim=2, SomYdim=2,
                                         SomReg0=800, SomReg1=400, SomSteps=2)
        ProtKerdensom.inputParticles.set(protOnlyAlign.outputParticles)
        self.launchProtocol(ProtKerdensom)
        self.assertIsNotNone(ProtKerdensom.outputClasses, "There was a problem with kerdensom")
        # self.validateFiles('ProtKerdensom', ProtKerdensom)



if __name__ == "__main__":
    pwtests.unittest.main()



# ***************************************************************************
# * Authors:    Daniel Marchán (da.marchan@cnb.csic.es)
# *
# *
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation; either version 3 of the License, or
# * (at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program; if not, write to the Free Software
# * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
# * 02111-1307  USA
# *
# *  All comments concerning this program package may be sent to the
# *  e-mail address 'scipion@cnb.csic.es'
# ***************************************************************************/
from pyworkflow.object import Pointer
from pyworkflow.tests import BaseTest, setupTestProject, DataSet
from pwem.protocols.protocol_import import ProtImportMicrographs
from pwem.protocols.protocol_boxsize_parameters import ProtBoxSizeParameters


class TestBoxSizeParameters(BaseTest):
    """ Test box size parameters protocol"""

    @classmethod
    def setData(cls):
        cls.ds = DataSet.getDataSet('relion_tutorial')

    @classmethod
    def setUpClass(cls):
        setupTestProject(cls)
        cls.setData()
        # Run needed protocols
        cls.runImportMicrograph()
        cls.runBoxSizeExtraction()

    @classmethod
    def runImportMicrograph(cls):

        """ Run an Import micrograph protocol. """
        protImport = cls.newProtocol(
            ProtImportMicrographs,
            samplingRateMode=0,
            filesPath=TestBoxSizeParameters.ds.getFile('micrographs/*.mrc'),
            samplingRate=3.54,
            magnification=59000,
            voltage=300,
            sphericalAberration=2)

        cls.launchProtocol(protImport)
        cls.protImport = protImport

    @classmethod
    def runBoxSizeExtraction(cls):
        """ Run box size related parameters on extraction boxSize. """
        protBoxSizeExtraction = cls.newProtocol(ProtBoxSizeParameters,
                                                inputMicrographs=cls.protImport.outputMicrographs,
                                                boxSize=46,
                                                boolExtractPartBx=True)
        protBoxSizeExtraction.inputMicrographs.set(cls.protImport.outputMicrographs)
        cls.launchProtocol(protBoxSizeExtraction)
        cls.protBoxSizeExtraction = protBoxSizeExtraction


    def testBoxSizeParametersExtractGautomatch(self):
        # No training mode picking, box size not provided by user
        prot = self._runBoxSizeParamsTestExtrGaut(label='Picking box size related parameters'
                                                                           ' extraction and gautomatch')

        self.assertEqual(prot.boxSizeExtraction, 102, "Estimated box size extraction does not match.")
        self.assertEqual(prot.radiusGautomatch, 180, "Estimated radius for "
                                                                        "gautomatch does not match.")
        self.assertEqual(prot.minIntPartDistanceGautomatch, 216, "Estimated min inter "
                                                                                    "particle distance for gautomatch "
                                                                                    "does not match.")
        self.assertEqual(prot.sigmaDiameterGautomatch, 288, "Estimated sigma diameter for "
                                                                               "gautomatch does not match.")
        self.assertEqual(prot.averageDiameterGautomatch, 362, "Estimated average diameter "
                                                                                 "for gautomatch does not match.")

    def testBoxSizeParametersRelionTopazConsensus(self):
        # No training mode picking, box size not provided by user
        prot = self._runBoxSizeParamsTestRelionTopazConsensus(label='Picking box size related parameters '
                                                                    'relion, topaz and radius consensus')

        self.assertEqual(prot.minLoGFilterRelion, 228, "Estimated minLoGFilterRelion for relion does not match.")
        self.assertEqual(prot.maxLoGFilterRelion, 252, "Estimated maxLoGFilterRelion for relion does not match.")
        self.assertEqual(prot.radiusTopaz, 30, "Estimated radius for topaz does not match.")
        self.assertEqual(prot.numPartPerImgTopaz, 300, "Estimated average diameter numPartPerImgTopaz "
                                                       "for topaz does not match.")
        self.assertEqual(prot.radiusConsensus, 62, "Estimated radius for picking consensus does not match.")

    def _runBoxSizeParamsTestExtrGaut(cls, label):
        protBoxSizeParams = cls.newProtocol(ProtBoxSizeParameters,
                                            inputMicrographs=cls.protImport.outputMicrographs,
                                            boolExtractPartBx=True,
                                            boolGautomatchParams=True)

        protBoxSizeParams.boxSize.setPointer(Pointer(cls.protBoxSizeExtraction, extended="boxSizeExtraction"))
        protBoxSizeParams.setObjLabel(label)
        cls.launchProtocol(protBoxSizeParams)

        return protBoxSizeParams

    def _runBoxSizeParamsTestRelionTopazConsensus(cls, label):
        protBoxSizeParams = cls.newProtocol(ProtBoxSizeParameters,
                                            inputMicrographs=cls.protImport.outputMicrographs,
                                            boolRelionParams=True,
                                            boolTopazParams=True,
                                            boolConsensusParams=True)

        protBoxSizeParams.boxSize.setPointer(Pointer(cls.protBoxSizeExtraction, extended="boxSizeExtraction"))
        protBoxSizeParams.setObjLabel(label)
        cls.launchProtocol(protBoxSizeParams)

        return protBoxSizeParams




# **************************************************************************
# *
# * Authors:     Pablo Conesa (pconesa@cnb.csic.es)
# *
# * Unidad de  Bioinformatica of Centro Nacional de Biotecnologia , CSIC
# *
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation; either version 3 of the License, or
# * (at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program; if not, write to the Free Software
# * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
# * 02111-1307  USA
# *
# *  All comments concerning this program package may be sent to the
# *  e-mail address 'scipion@cnb.csic.es'
# *
# **************************************************************************
from pyworkflow.gui import showInfo
from pwem.viewers import EmPlotter
from pyworkflow.viewer import Viewer
import pwem.objects as emobj

class AngularDistributionViewer(Viewer):
    """ Visualize the output of protocol reconstruct swarm """
    _label = 'Score transformation viewer'
    _targets = [emobj.SetOfParticles, emobj.SetOfVolumes]

    @staticmethod
    def plotAngularDistribution(emSet:emobj.EMSet, histogram=False):

        plotter = EmPlotter(x=1, y=1, windowTitle='Angular distribution')

        plotter.plotAngularDistributionFromSet(emSet, "Angular distribution", histogram=histogram)

        return plotter

    def _visualize(self, outputSet: emobj.EMSet, **kwargs):
        # Keep input object in case we need to launch
        # a new protocol and set dependencies

        views =[]

        # Weird plot. Not sure if used at all:
        # views.append(self.plotAngularDistribution(outputSet, histogram=True))

        firstItem = outputSet.getFirstItem()

        if not firstItem.hasTransform():
            showInfo("Missing alignment information", "This set does not have alignment information.")
            return

        views.append(self.plotAngularDistribution(outputSet))

        return views




